package error

import (
	"errors"
	"fyne.io/fyne/v2/container"
	"fyne.io/fyne/v2/lang"
	"fyne.io/fyne/v2/widget"
	"git.kor-elf.net/kor-elf/gui-for-ffmpeg/kernel"
	"git.kor-elf.net/kor-elf/gui-for-ffmpeg/localizer"
	"github.com/nicksnyder/go-i18n/v2/i18n"
	"go.etcd.io/bbolt"
)

type ViewContract interface {
	PanicError(err error)
}

type View struct {
	app           kernel.AppContract
	isSetLanguage bool
}

func NewView(app kernel.AppContract) *View {
	return &View{
		app:           app,
		isSetLanguage: true,
	}
}

func (v View) PanicError(err error) {
	if v.isSetLanguage {
		v.isSetLanguage = false
		_ = v.app.GetLocalizerService().SetCurrentLanguageByCode(lang.SystemLocale().LanguageString())
	}

	messageHead := v.app.GetLocalizerService().GetMessage(&i18n.LocalizeConfig{
		MessageID: "error",
	})

	messagetText := err.Error()
	if errors.Is(err, bbolt.ErrTimeout) {
		messagetText = v.app.GetLocalizerService().GetMessage(&i18n.LocalizeConfig{
			MessageID: "errorDatabaseTimeout",
		})
	}

	v.app.GetWindow().SetContent(container.NewBorder(
		container.NewVBox(
			widget.NewLabel(messageHead),
			widget.NewLabel(messagetText),
		),
		nil,
		nil,
		nil,
		localizer.LanguageSelectionForm(v.app.GetLocalizerService(), func(lang kernel.Lang) {
			v.PanicError(err)
		}),
	))
}

func (v View) PanicErrorWriteDirectoryData() {
	if v.isSetLanguage {
		v.isSetLanguage = false
		_ = v.app.GetLocalizerService().SetCurrentLanguageByCode(lang.SystemLocale().LanguageString())
	}

	message := v.app.GetLocalizerService().GetMessage(&i18n.LocalizeConfig{
		MessageID: "errorDatabase",
	})
	messageHead := v.app.GetLocalizerService().GetMessage(&i18n.LocalizeConfig{
		MessageID: "error",
	})

	v.app.GetWindow().SetContent(container.NewBorder(
		container.NewVBox(
			widget.NewLabel(messageHead),
			widget.NewLabel(message),
		),
		nil,
		nil,
		nil,
		localizer.LanguageSelectionForm(v.app.GetLocalizerService(), func(lang kernel.Lang) {
			v.PanicErrorWriteDirectoryData()
		}),
	))
}
